{:Procedimientos genricos.
@author Neftal -Germn Estvez-
}
unit uProcedures;
{=========================================================================

  Procedimientos genricos

  Unit

========================================================================
  Historia de las Versiones
------------------------------------------------------------------------

  25/01/2011     Neftal -Germn Estvez-
  * Creacin.
------------------------------------------------------------------------
   ...

=========================================================================

  Errores detectados no corregidos

=========================================================================}

//=========================================================================
//
// I N T E R F A C E
//
//=========================================================================
interface

uses
  SHDocVw, DB, Classes, Graphics,
  UConstantes,
  SysUtils;

type
  {: Array de cadenas}
  TArrString = array of string;

  //: Estructura para la bsqueda de recurso
  PResFinder = ^TResFinder;
  TResFinder = record
      ResName:PChar;      // Nombre de recurso que se est buscando.
      ResType:PChar;      // Tipo de recurso que se est buscando.
      Instance:THandle;   // Instancia donde se ha encontrado.
    end;

// distance in meters onto the WGS84 geoid between two Coordinates
// some data are exageretely precise (way beyond GPS's capacity) so I
// added the AlmostNul switch, as arccos(1) returns an error when two points
// are too close (and therefore distance is 0)
function WGS84Distance(plon1,plat1,plon2,plat2:Double; Almostnul:boolean=False):double;

Function ISO8601ToDateTime(Value:String):TDateTime;


{$IFDEF VER220}
function GetFmtS():TFormatSettings;
{$ENDIF}
function RGBToWebColorStr(RGB: Integer): string;

{:Rutina callback para la enumeracin de mdulos cargados.}
function _MyEnumResourceModules (Instance:{$IFDEF VER230}NativeInt{$ELSE}Pointer{$ENDIF};
                                 Data:Pointer):Boolean;
//: Buscar la instancia donde est un determinado recurso.
function FindInstanceOfResource(ResourceName:string;
                                ResourceType:PChar;
                                out Instance:THandle):Boolean;

procedure ConvertTrackToKLM(ATrackName:string;
                            ATrackDesc:string;
                            dsPoints:TDataSet;
                            latFieldName, lonFieldName, eleFieldName, timeFieldName:string;
                            var TSTrack:TStrings;
                            ATrackColor:TColor=clBlue;
                            ATrackWidth:integer=2);

procedure _GetMapCenter(cdsPoints:TDataset; var LatC, LonC:Double);

procedure _GetPoints(cdsPoints:TDataset; AList:TStrings);

//...................................................
// Generacin de mapas con Google maps
procedure VisualizarMapa(wbControl:TWebBrowser;
                         Lat,Lon:string;
                         AMapaHeight, AMapaWidth:integer;
                         AUserZoom:integer=14; AUserType:string=G_NORMAL_MAP;
                         AListPoints:String=STR_EMPTY);
procedure LoadMapa(wbControl:TWebBrowser;
                   H, W:integer; AControls:TArrString;
                   Lat,Lon:string; AZoom:Integer;
                   AVisibleMark, AVisibleWindow:Boolean;
                   AMapType:string=STR_EMPTY;
                   AListPoints:String=STR_EMPTY);




//=========================================================================
//
// I M P L E M E N T A T I O N
//
//=========================================================================
implementation

uses
  Forms, Windows, Dialogs, DateUtils, Math, StrUtils;

function RGBToWebColorStr(RGB: Integer): string;
begin
  Result := UpperCase(Format('#%.2x%.2x%.2x', [GetRValue(RGB),
    GetGValue(RGB), GetBValue(RGB)]));  { do not localize }
end;


{$IFDEF VER220}
function GetFmtS():TFormatSettings;
begin
  GetLocaleFormatSettings(LOCALE_SYSTEM_DEFAULT, Result);
  Result.DecimalSeparator := '.';
end;
{$ENDIF}

// distance in meters onto the WGS84 geoid between two Coordinates
// some data are exageretely precise (way beyond GPS's capacity) so I
// added the AlmostNul switch, as arccos(1) returns an error when two points
// are too close (and therefore distance is 0)
function WGS84Distance(plon1,plat1,plon2,plat2:Double; Almostnul:boolean=False):double;
var
  d,s,c : Double;
  lat1 : Double;
  lon1 : Double;
  lat2 : Double;
  lon2 : Double;
  almost1 : double;
Const
  R = 6378137; // Medium earth radius in meter
begin
  // d = R x arcos [ sin(lat1) x sin(lat2) + cos(lat1) x cos(lat2) x cos(lon2-lon1) ]
  lat1 := DegToRad(plat1);
  lon1 := DegToRad(plon1);
  lat2 := DegToRad(plat2);
  lon2 := DegToRad(plon2);
  if (lon1 = lon2) AND
     (lat1 = lat2) then
    d := 0
  else
  begin
    s := sin(lat1)*sin(lat2);
    c := cos(lat1)*cos(lat2)*cos(lon2-lon1);

    if Almostnul then
      almost1 := round((s+c)*100000000)/100000000
    else
      almost1 := (s+c);

    if almost1 <> 1 then
      d := R*arccos(s+c)
    else
      d := 0;
  end;
  Result := d;
end;


Function ISO8601ToDateTime(Value : String) : TDateTime;
var
  prt : string;
  dpart, tpart, tzpart : String;
  Quand : TDateTime;
  y,m,d,h,n,s,ms : word;
  w : integer;
  tzh,tzm : integer;  // time zone bias if not zulu time
  tzpositive : boolean;
  dcm : Char;
  f : Double;
begin
  // 2004-11-26T14:42:38Z
  // 2004-11-26 14:42:38+0100
  // 2004-W19-2 14:42:38.010+03:30
  // See http://en.wikipedia.org/wiki/ISO_8601

  Result := 0;
  if length(Value) < 8 then
    Exit;

  y := 1900;
  m := 1;
  d := 1;
  n := 0;
  s := 1;
  ms := 0;
  tzh := 0;
  tzm := 0;
  tzpositive := True;

  Value := Trim(Value);

  if pos('T',Value)>0 then
  begin
    dpart := copy(Value,1,pos('T',Value)-1);
    tpart := copy(Value,pos('T',Value)+1,length(Value));
  end
  else if pos(' ',Value)>0 then
  begin
    dpart := copy(Value,1,pos(' ',Value)-1);
    tpart := copy(Value,pos(' ',Value)+1,length(Value));
  end
  else
  begin
    dpart := Value;
    tpart := '';
  end;

  if length(dpart) < 4 then
    Exit;

  if pos('Z',tpart)>0 then
  begin
    tpart := copy(tpart,1,pos('Z',tpart)-1);
    tzpart := 'Z';
  end else if pos('+',tpart)>0 then
  begin
    tzpart := copy(tpart,pos('+',tpart),length(tpart));
    tpart := copy(tpart,1,pos('+',tpart)-1);
  end else if pos('-',tpart)>0 then
  begin
    tzpart := copy(tpart,pos('-',tpart),length(tpart));
    tpart := copy(tpart,1,pos('-',tpart)-1);
  end else
  begin
    tzpart := '';
  end;

  dcm :=
  {$IFDEF VER220}
		GetFmtS.DecimalSeparator;
  {$ELSE}
    DecimalSeparator;
  {$ENDIF}

  if pos(',',tpart) > 0 then
    tpart := StringReplace(tpart,',',dcm,[]);

  //the standard also permits the expansion of the year representation [YYYYY], but only by prior agreement between the sender and the receiver.
  // so in GPX it will ALWAYS begin with YYYY

  prt := copy(dpart,1,4);
  y := StrToIntDef(prt,1900);
  dpart := copy(dpart,5,length(dpart));
  if length(dpart) > 0 then // there is more than just a year
  begin
    if dpart[1] = '-' then
      dpart := copy(dpart,2,length(dpart)-1);

    if (length(dpart) = 3) and (dpart[1]<>'W') then // ordinal date YYYYDDD & YYYY-DDD
    begin
      d := StrTointDef(dpart,1);
      Quand := EncodeDateDay(y,d);
      DecodeDate(Quand,y,m,d);
    end else if (dpart[1]='W') then // Week date YYYYWww or YYYY-Www & YYYY-Www-D or YYYYWwwD
    begin
      dpart := copy(dpart,2,length(dpart)-1);
      prt := copy(dpart,1,2);
      w := StrToIntDef(prt,1);
      d := 1;
      if length(dpart) > length(prt) then // YYYY-Www-D or YYYYWwwD
      begin
        dpart := copy(dpart,3,length(dpart));
        if dpart[1] = '-' then
          dpart := copy(dpart,2,length(dpart)-1);
        d := StrToIntDef(dpart,1);
      end;
      Quand := EncodeDateWeek(y,w,d);
      DecodeDate(Quand,y,m,d);
    end else // YYYY-MM-DD or YYYYMMDD or YYYY-MM
    begin
      prt := copy(dpart,1,2);
      m := StrToIntDef(prt,1);
      if length(dpart) > length(prt) then // YYYY-MM-DD or YYYYMMDD
      begin
        dpart := copy(dpart,3,length(dpart));
        if dpart[1] = '-' then
          dpart := copy(dpart,2,length(dpart)-1);
        d := StrToIntDef(dpart,1);
      end;
    end
  end;

  prt := copy(tpart,1,2);
  h := StrToIntDef(prt,0);
  tpart := copy(tpart,3,length(tpart));
  if length(tpart)>1 then // hh:mm:ss or hhmmss & hh:mm or hhmm & hh & hh.xxxx
  begin
    if tpart[1] = ':' then
      tpart := copy(tpart,2,length(tpart)-1);

    if tpart[1] = dcm then // hh.xxxx
    begin
      {$IFDEF VER220}
      f := StrToFloatDef('0'+tpart,0,GetFmtS);
      {$ELSE}
      f := StrToFloatDef('0'+tpart,0);
      {$ENDIF}

      f := f * 60;
      n := trunc(f);
      f := f - n;
      f := f * 60;
      s := trunc(f);
      f := f - s;
      f := f *1000;
      ms := trunc(f);
    end else
    begin
      prt := copy(tpart,1,2);
      n := StrToIntDef(prt,0);
      tpart := copy(tpart,3,length(tpart));

      if length(tpart)>1 then // hh:mm:ss or hhmmss & hh:mm.xxxx or hhmm.xxxx
      begin
        if tpart[1] = ':' then
          tpart := copy(tpart,2,length(tpart)-1);

        if tpart[1] = dcm then // hh:mm.xxxx
        begin
          {$IFDEF VER220}
          f := StrToFloatDef('0'+tpart,0,GetFmtS);
          {$ELSE}
          f := StrToFloatDef('0'+tpart,0);
          {$ENDIF}
          f := f * 60;
          s := trunc(f);
          f := f - s;
          f := f *1000;
          ms := trunc(f);
        end else
        begin
          prt := copy(tpart,1,2);
          s := StrToIntDef(prt,0);
          tpart := copy(tpart,3,length(tpart));

          if length(tpart)>1 then // hh:mm:ss.xxx or hhmmss.xxx
          begin
            if tpart[1] = dcm then // hh.xxxx
            begin
              {$IFDEF VER220}
              f := StrToFloatDef('0'+tpart,0,GetFmtS);
              {$ELSE}
              f := StrToFloatDef('0'+tpart,0);
              {$ENDIF}
              f := f *1000;
              ms := trunc(f);
            end;
          end;
        end;
      end;
    end;
  end;

  if length(tzpart) < 3 then // nothing or Z
  begin
    tzpart := '';
  end
  else // hh:mm or hhmm & hh
  begin
    tzpositive := not (tzpart[1] = '-');

    tzpart := copy(tzpart,2,length(tzpart)-1);
    prt := copy(tzpart,1,2);
    tzh := StrToIntDef(prt,0);
    if length(tzpart) > length(prt) then // hh:mm or hhmm
    begin
      tzpart := copy(tzpart,3,length(tzpart)-1);
      if tzpart[1] = ':' then
        tzpart := copy(tzpart,2,length(tzpart)-1);
      tzm := StrToIntDef(tzpart,0);
    end;
  end;

  if tzpositive then
    Quand := EncodeDate(y,m,d) + EncodeTime(h,n,s,ms) + EncodeTime(tzh,tzm,0,0)
  else
    Quand := EncodeDate(y,m,d) + EncodeTime(h,n,s,ms) - EncodeTime(tzh,tzm,0,0);

  Result := Quand;
end;

//: Visualizar el mapa segun los datos.
procedure VisualizarMapa(wbControl:TWebBrowser;
                         Lat,Lon:string;
                         AMapaHeight, AMapaWidth:integer;
                         AUserZoom:integer=14;
                         AUserType:string=G_NORMAL_MAP;
                         AListPoints:String=STR_EMPTY);
const
  COLOR_VERDE = $00D2E6D2;
  COLOR_ROJO = $00CACAFF;
var
  arr:TArrString;
begin

  // Elementos a visualizar en el mapa
  SetLength(arr, 2);
  arr[0] := MAP_LARGE_CONTROL;
  arr[1] := MAP_TYPE_CONTROL;

  // Ini
  if (AUserZoom = 0) then begin
    AUserZoom := 14;
  end;

  // algo de info
  if ((Lat <> STR_EMPTY) and (Lon <> STR_EMPTY)) or
     (AListPoints <> STR_EMPTY) then begin
    // Code := 200;
  end
  else begin
    // error ni Lat,Lon rellenadas ni ruta
    Exit;
  end;

  // Cargar mapa inicial
  LoadMapa(wbControl,
           AMapaHeight - 25, AMapaWidth - 25, arr,
           Lat, Lon, AuserZoom,
           True, False{True}, AuserType,
           AListPoints);
end;

//: Carga el mapa en el formulario.
procedure LoadMapa(wbControl:TWebBrowser;
                  H, W: integer; AControls: TArrString;
                  Lat, Lon: string; AZoom:Integer;
                  AVisibleMark, AVisibleWindow: Boolean;
                  AMapType:string=STR_EMPTY;
                  AListPoints:String=STR_EMPTY);
const
  EBLOCK = 'TFormVerMapa._LoadMapa';
var
  TSWeb:TStrings;
  FNameWeb:string;


  //
  procedure _AddMarca();
  const
    STR_PREFIX_MARCA = '//##MARCA##';
    STR_MARCA1 = '        var point = new GLatLng(%s,%s);';
    STR_MARCA2 = '        var marker = new GMarker(point, {draggable: true});';
    STR_MARCA3 = '        map.addOverlay(marker);';
  var
    i, j:Integer;
    str, line:string;
  begin

    // Buscar la cadena
    for i := 0 to (TSWeb.Count - 1) do begin
      line := TSWeb[i];
      j := AnsiPos(STR_PREFIX_MARCA, line);
      // Es la lnea
      if (j > 0) then begin
        if (AVisibleMark) then begin
          Str := Format(STR_MARCA1,[Lat, Lon]);
          TSWeb[i] := Str;
          TSWeb.Insert(i+1, STR_MARCA3);
          TSWeb.Insert(i+1, STR_MARCA2);
        end
        else begin
          TSWeb[i] := STR_EMPTY;
        end;

        // Grabar
        TSWeb.SaveToFile(FNameWeb);

        Break;
      end;
    end;
  end;
  //
  procedure _AddTrack();
const
    STR_PREFIX_TRACK  = '//##TRACK##';
    STR_TRACKCOMMENT  = '        // Puntos de la ruta';
    STR_TRACK_CAB     = '        var polyline = new GPolyline([';
    STR_LINESPACE     = '       ';
    STR_PIE1          = '        ], "#ff0000", 5);';
    STR_PIE2          = '        map.addOverlay(polyline);';
    STR_PIE3          = '        // map.setZoom(map.getBoundsZoomLevel(polyline.getBounds()));';
  var
    i, j:Integer;
    line:string;
  begin

    // Buscar la cadena
    for i := 0 to (TSWeb.Count - 1) do begin
      line := TSWeb[i];
      j := AnsiPos(STR_PREFIX_TRACK, line);
      // Es la lnea
      if (j > 0) then begin

        TSWeb[i] := STR_TRACKCOMMENT;
        TSWeb.Insert(i+1, STR_PIE3);
        TSWeb.Insert(i+1, STR_PIE2);
        TSWeb.Insert(i+1, STR_PIE1);
        TSWeb.Insert(i+1, AListPoints);
        TSWeb.Insert(i+1, STR_TRACK_CAB);

        Break;
      end;
    end;
  end;
  //
  procedure _AddVentana();
  const
    STR_PREFIX_VENTANA = '//##VENTANA##';
    STR_VENTANA = '       marker.openInfoWindowHtml(html);';
  var
    i, j:Integer;
    line:string;
  begin

    // Buscar la cadena
    for i := 0 to (TSWeb.Count - 1) do begin
      line := TSWeb[i];
      j := AnsiPos(STR_PREFIX_VENTANA, line);
      // Es la lnea
      if (j > 0) then begin
        if (AVisibleWindow) then begin
          TSWeb[i] := STR_VENTANA;
        end
        else begin
          TSWeb[i] := STR_EMPTY;
        end;

        // Grabar
        TSWeb.SaveToFile(FNameWeb);

        Break;
      end;
    end;
  end;
  //
  procedure _ChangeSize();
  const
    STR_PREFIX_TAM = '//##TAMANYO##';
    // STR_TAM = '<div id="map" style="width: %dpx; height: %dpx"></div>';
    STR_TAM = '<div id="map" style="position: absolute; top: 0; left: 0; width: 100%; height: 100%;"></div>';
  var
    i, j:Integer;
    str, line:string;
  begin

    // Buscar la cadena
    for i := 0 to (TSWeb.Count - 1) do begin
      line := TSWeb[i];
      j := AnsiPos(STR_PREFIX_TAM, line);
      // Es la lnea
      if (j > 0) then begin
  //      Str := Format(STR_TAM,[W, H]);
        Str := STR_TAM;
        TSWeb[i] := Str;


        TSWeb.SaveToFile(FNameWeb);

        Break;
      end;
    end;
  end;
  //
  procedure _ChangeMapType();
  const
    STR_PREFIX_TYPE = '//##MAPTYPE##';
    STR_ADD  = '        map.addMapType(G_PHYSICAL_MAP); ';
    STR_TYPE = '        map.setMapType(%s);';
  var
    i, j:Integer;
    str, line:string;
  begin

    // Buscar la cadena
    for i := 0 to (TSWeb.Count - 1) do begin
      line := TSWeb[i];
      j := AnsiPos(STR_PREFIX_TYPE, line);
      // Es la lnea
      if (j > 0) then begin
        Str := Format(STR_TYPE,[AMapTYpe]);
        TSWeb[i] := Str;
        // Aadir el tipo HIBID
        TSWeb.Insert(i + 1, STR_ADD);
        // Grabas
        TSWeb.SaveToFile(FNameWeb);
        Break;
      end;
    end;
  end;
  //
  procedure _SetCenter();
  const
    STR_PREFIX_TYPE = '//##SETCENTER##';
    STR_TYPE = '        map.setCenter(new GLatLng(%s,%s),%d);';
  var
    i, j:Integer;
    str, line:string;
  begin

    // Vacos?
    if (Lat = STR_EMPTY) and (Lon = STR_EMPTY) then begin
      Exit;
    end;

    // Buscar la cadena
    for i := 0 to (TSWeb.Count - 1) do begin
      line := TSWeb[i];
      j := AnsiPos(STR_PREFIX_TYPE, line);
      // Es la lnea
      if (j > 0) then begin
        Str := Format(STR_TYPE,[Lat, Lon, AZoom]);
        TSWeb[i] := Str;
        TSWeb.SaveToFile(FNameWeb);

        Break;
      end;
    end;
  end;
  //
  procedure _AddControls();
  const
    STR_PREFIX_CONTROLS = '//##CONTROLS##';
    STR_CONTROL_STRING = '        map.addControl(new %s());';
  var
    i, j, k:Integer;
    str, line:string;
  begin
    // Buscar la cadena
    for i := 0 to (TSWeb.Count - 1) do begin
      line := TSWeb[i];
      j := AnsiPos(STR_PREFIX_CONTROLS, line);
      // Es la lnea
      if (j > 0) then begin
        // Eliminar
        TSWeb.Delete(i);

        // Aadir los controles
        for k := 0 to (Length(AControls)-1) do begin
          Str := AControls[k];
          TSWeb.Insert(i, Format(STR_CONTROL_STRING,[Str]));
        end;

        // Grabar
        TSWeb.SaveToFile(FNameWeb);

        Break;
      end;
    end;
  end;
  //
  //: Recupera el texto de la pgina desde el recurso.
  function _RetrievePageText():string;
  const
    EBLOCK = '_RetrievePageText';
  var
    rs: TResourceStream;
    tf: TStrings;
    b:Boolean;
    resName:String;
  begin
    // Se trata de seguridad de Visualizacion?
    resName := RES_HTML_PAGEMAP;

    // Llamar a la bsqueda de recurso
    b := FindInstanceOfResource(resName, RT_RCDATA, HInstance);

    if not b then begin
      // Error de recurso no encontrado
      raise Exception.Create(EMAP_MSG_ERRORGETRES);
//      Application.Terminate;
    end;

    // Crear el StringList
    tf := TStringList.Create;
    rs := TResourceStream.Create(HInstance, resName, RT_RCDATA);
    // proteccion para liberar
    try
      // proteccion
      try
        // Cargar el recurso
        tf.LoadFromStream(rs);
      except
        // Error recurso no encontrado (al recuperar)
        raise Exception.Create(EMAP_MSG_ERRORGETRES);
        // Application.Terminate;
      end;
      Result := tf.Text;
    // liberar
    finally
      rs.Free;
      tf.Free;
    end;
  end;
  //

begin

//  // desactivar el Timer
//  timerNew.Enabled := False;

  // ini
  FNameWeb := ExtractFilePath(Application.ExeName) + '1234.tmp';

  TSWeb := TStringList.Create();
  // Para liberar
  try

    // Crgar la pgina en memoria
    //--TSWeb.LoadFromFile(ExtractFilePath(ParamStr(0)) + '_Mapa.html');
    try
      TSWeb.Text := _RetrievePageText();
    except
      MessageDlg('No se ha podido acceder a la pgina para mostrar el mapa.', mtError, [mbOK], 0);
      Exit;
    end;

    // proteccion para guardar el fichero
    try

      // Ajustar tamao
      _ChangeSize();
      _AddControls();
      _ChangeMapTYpe();
      // no aadimos marca
      //_AddMarca();

      // Hay ruta?
      if (AListPoints <> STR_EMPTY) then begin
        _AddTrack();
      end;

      _AddVentana();
      _SetCenter();

      // DEBUG
      TSWeb.SaveToFile(ExtractFilePath(Application.ExeName) + 'page.html');

      // Load the HTML page
      wbControl.Navigate(FNameWeb);

    except
      on E:EFCreateError do begin
        MessageDlg(Format('Se ha producido un error crear el mapa. Consulta con el Administrador del sistema. '+ CHAR_CRLF + 'Error: %s',[E.Message]), mtError, [mbOK], 0);
//--        Self.Close;
      end;
      on E:Exception do begin
        MessageDlg(Format('Se ha producido un error crear al generar el mapa. Consulta con el Administrador del sistema. '+ CHAR_CRLF + 'Error: %s',[E.Message]), mtError, [mbOK], 0);
//--        Self.Close;
      end;
    end;
  finally
    FreeAndNil(TSWeb);
 //   DeleteFile(PChar(FNameWeb));
  end;
end;


{:Rutina callback para la enumeracin de mdulos cargados.
  Rutina callback para la enumeracin de mdulos cargados (el exe y los packages
  estticos y dinmicos)
}
function _MyEnumResourceModules (Instance:{$IFDEF VER230}NativeInt{$ELSE}Pointer{$ENDIF};
                                 Data:Pointer):Boolean;
begin
  // Existe el recursos en esta instancia ?
  if FindResource(Instance, PResFinder(Data).ResName, PResFinder(Data).ResType) <> 0 then begin
    // Apuntarse la instancia donde se ha encontrado
    PResFinder(Data).Instance := Instance;
    // Devolver False para detener la enumeracin de instancias
    Result := False;
  end
  else begin
    // Devolver True para continuar la enumeracin
    Result := True;
  end;

end;

{:Buscar la instancia donde est un determinado recurso.
  Cuando una aplicacin usa packages, un recurso puede estar en cualquiera de
  esos packages .BPLs o en el propio ejecutable .EXE, esta rutina nos indicar
  en que instancia se encuentra un determinado recurso.
}
function FindInstanceOfResource ( ResourceName:string;
                                  ResourceType:PChar;
                                  out Instance:THandle):Boolean;
var
  ResFinder:TResFinder;
begin

  // Inicializar estructura de bsqueda de recurso
  ResFinder.ResName := PChar(ResourceName);
  ResFinder.ResType := ResourceType;
  ResFinder.Instance := 0;

  // Para cada mdulo cargado, llamar a '_MyEnumResourceModules'
  EnumResourceModules (_MyEnumResourceModules, @ResFinder);

  // Se ha encontrado alguna instancia ?
  if ResFinder.Instance <> 0 then begin
    // Asignar la instancia encontrada
    Instance := ResFinder.Instance;
    // Devolver True
    Result := True;
  end
  else begin
    // Por seguridad, asignar 0
    Instance := 0;
    // Devolver False
    Result := False;
  end;
end;


{ Generar un fichero de Track en formato KML. }
procedure ConvertTrackToKLM(ATrackName:string;
                            ATrackDesc:string;
                            dsPoints:TDataSet;
                            latFieldName, lonFieldName, eleFieldName, timeFieldName:string;
                            var TSTrack:TStrings;
                            ATrackColor:TColor=clBlue;
                            ATrackWidth:integer=2);


const
  CAB_XML         = '<?xml version="1.0" encoding="UTF-8"?>';
  CAB_KLM         = '<kml xmlns="http://earth.google.com/kml/2.1">';
  CAB_PLACEMARK   = '<Placemark>';
	CAB_NAME        = '<name>%s</name>';
	STR_STYLE       = '<Style><LineStyle><color>%s</color>' +
                    '<width>%d</width></LineStyle></Style>';
	CAB_LINE        = '<LineString><coordinates>';
//  ... coordenadas ...
  PIE_LINE        = '</coordinates></LineString>';
  PIE_PLACEMARK   = '</Placemark>';
  PIE_KLM         = '</kml>';
var
  StrLine, Str:string;
begin
  // Limpiar
  TSTrack.Clear;
  // Aadir cabeceras
  TSTrack.Add(CAB_XML);
  TSTrack.Add(CAB_KLM);
  TSTrack.Add(CAB_PLACEMARK);
  // Datos de la ruta
  TSTrack.Add(Format(CAB_NAME,[AtrackName]));

  // color
  {$IFDEF VER220}
  Str := 'ff' + RGBToWebColorStr(ATrackColor); // transparencia + color
  {$ELSE}
  Str := 'ff' + uProcedures.RGBToWebColorStr(ATrackColor);
  {$ENDIF}
  Str := AnsiReplaceText(Str, '#', '');
  TSTrack.Add(Format(STR_STYLE,[Str, ATrackWidth])); // TrackColor en HEXA
  TSTrack.Add(CAB_LINE);

  // recorrido de los puntos
  //dsPoints := FormData.cdsPoints;
  dsPoints.First;
  // recorrido
  while not (dsPoints.Eof) do begin
    // Info. de punto
    StrLine := Copy(dsPoints.FieldByName(lonFieldName).AsString,1,10) + ',';
    StrLine := StrLine + Copy(dsPoints.FieldByName(latFieldName).AsString, 1,10) +  ',';
    StrLine := StrLine + dsPoints.FieldByName(eleFieldName).AsString + ' ';
    // Aadir la linea
    TSTrack.Add(StrLine);
    // Next
    dsPoints.Next;
  end;

  TSTrack.Add(PIE_LINE);
  TSTrack.Add(PIE_PLACEMARK);
  TSTrack.Add(PIE_KLM);

end;


procedure _GetMapCenter(cdsPoints:TDataset; var LatC, LonC:Double);
var
  c1, c2:Double;
  count:integer;
  Str:String;
begin
  // ini
  c1 := 0;
  c2 := 0;
  count := 0;
  // Lista de puntos
  cdsPoints.First;
  // Recorrer los puntos
  While not (cdsPoints.Eof) do begin
    Str := Copy(cdsPoints.FieldByName('lat').AsString,1,10);
    Str := AnsiReplaceText(Str, CHAR_POINT, CHAR_COMMA);
    c1 := c1 + StrToFloat(Str);

    Str := Copy(cdsPoints.FieldByName('lon').AsString,1,10);
    Str := AnsiReplaceText(Str, CHAR_POINT, CHAR_COMMA);
    c2 := c2 + StrToFloat(Str);

    // Siguiente
    cdsPoints.Next;
    Inc(Count);
  end;
  // media
  LatC := c1 / count;
  LonC := c2 / count;
end;

procedure _GetPoints(cdsPoints:TDataset; AList:TStrings);
const
  STR_NEWPOINT  = 'new GLatLng(%s,%s)';
  STR_SPACE     = '          ';
var
  Str1, Str2, Line:String;
  first:boolean;
begin
  // ini
  AList.Clear;
  first := True;
  // Lista de puntos
  cdsPoints.First;
  cdsPoints.DisableControls;
  try
    // Recorrer los puntos
    While not (cdsPoints.Eof) do begin
      Str1 := Copy(cdsPoints.FieldByName('lat').AsString,1,12);
      Str2 := Copy(cdsPoints.FieldByName('lon').AsString,1,12);

      Line := Format(STR_NEWPOINT, [Str1, Str2]);

      if (first) then begin
        first := False;
      end
      else begin
        Line := ',' + Line;
      end;

      AList.Add(STR_SPACE + Line);

      // Siguiente
      cdsPoints.Next;
    end;
  finally
    cdsPoints.EnableControls;
  end;
end;

end.
